@@ -17,6 +17,7 @@
 	return(length(unique(x)))
 	}
 
+
 .removeEdgeDuplicates <- function(x, weights = NA) # x is an edge list.
 	{ 
 	x <- cbind(as.character(x[,1]),as.character(x[,2]))
@@ -54,7 +55,7 @@
 	}
 
 
-getLinkCommunities <- function(x, plot = TRUE, hcmethod = "average", edglim = 10^4, directed = FALSE, dirweight = 0.5, removetrivial = TRUE) 
+getLinkCommunities <- function(x, hcmethod = "average", edglim = 10^4, directed = FALSE, dirweight = 0.5, plot = TRUE, removetrivial = TRUE, verbose = TRUE) 
 	# x is an edge list. Nodes can be ASCII names or integers, but are always treated as character names in R.
 	# If plot is true (default), a dendrogram and partition density score as a function of dendrogram height are plotted side-by-side.
 	# When there are more than "edglim" edges, hierarchical clustering is carried out via temporary files written to disk using compiled C++ code.
@@ -68,7 +69,9 @@ getLinkCommunities <- function(x, plot = TRUE, hcmethod = "average", edglim = 10
 	}else{
 		stop("\nInput data must be an edge list.\n")
 		}
-	cat("   Removing loops...\n")
+	if(verbose){
+		cat("   Removing loops...\n")
+		}
 	loops <- apply(xx,1,.getLoopInds)
 	if(length(which(loops==1))>0){
 		xx <- xx[-which(loops==1),]
@@ -76,7 +79,9 @@ getLinkCommunities <- function(x, plot = TRUE, hcmethod = "average", edglim = 10
 			wt <- wt[-which(loops==1)]
 			}
 		}
-	cat("   Removing edge duplicates...\n")
+	if(verbose){
+		cat("   Removing edge duplicates...\n")
+		}
 	if(length(wt)>1){
 		yy <- .removeEdgeDuplicates(xx, weights = wt)
 		xx <- yy$edges
@@ -105,26 +110,28 @@ getLinkCommunities <- function(x, plot = TRUE, hcmethod = "average", edglim = 10
 		disk <- FALSE
 		emptyvec <- rep(1,(len*(len-1))/2)
 		if(length(wt)>1){ weighted <- TRUE}else{ wt <- 0; weighted <- FALSE}
-		dissvec <- .C("getEdgeSimilarities",as.integer(edges[,1]),as.integer(edges[,2]),as.integer(len),rowlen=integer(1),weights=as.double(wt),as.logical(directed),as.double(dirweight),as.logical(weighted),as.logical(disk), dissvec = as.double(emptyvec), as.logical(carriageret))$dissvec
+		dissvec <- .C("getEdgeSimilarities",as.integer(edges[,1]),as.integer(edges[,2]),as.integer(len),rowlen=integer(1),weights=as.double(wt),as.logical(directed),as.double(dirweight),as.logical(weighted),as.logical(disk), dissvec = as.double(emptyvec), as.logical(carriageret), as.logical(verbose))$dissvec
 		distmatrix <- matrix(1,len,len)
 		distmatrix[lower.tri(distmatrix)] <- dissvec
 		colnames(distmatrix) <- 1:len
 		rownames(distmatrix) <- 1:len
 		distobj <- as.dist(distmatrix) # Convert into 'dist' object for hclust.
 		rm(distmatrix)
-		cat("\n   Hierarchical clustering of edges...")
+		if(verbose){
+			cat("\n   Hierarchical clustering of edges...")
+			}
 		hcedges <- hclust(distobj, method = hcmethod)
 		hcedges$order <- rev(hcedges$order)
-		cat("\n")
+		if(verbose){cat("\n")}
 		#return(hcedges)
 	}else{
 		disk <- TRUE
 		if(length(wt)>1){ weighted <- TRUE}else{ wt <- 0; weighted <- FALSE}
-		rowlen <- .C("getEdgeSimilarities",as.integer(edges[,1]),as.integer(edges[,2]),as.integer(len),rowlen=integer(len-1),weights=as.double(wt),as.logical(directed),as.double(dirweight),as.logical(weighted),as.logical(disk), dissvec = double(1), as.logical(carriageret))$rowlen
+		rowlen <- .C("getEdgeSimilarities",as.integer(edges[,1]),as.integer(edges[,2]),as.integer(len),rowlen=integer(len-1),weights=as.double(wt),as.logical(directed),as.double(dirweight),as.logical(weighted),as.logical(disk), dissvec = double(1), as.logical(carriageret), as.logical(verbose))$rowlen
 		#return(rowlen)
-		cat("\n")
-		hcobj <- .C("hclustLinkComm",as.integer(len),as.integer(rowlen),heights = single(len-1),hca = integer(len-1),hcb = integer(len-1), as.logical(carriageret))
-		cat("\n")
+		if(verbose){cat("\n")}
+		hcobj <- .C("hclustLinkComm",as.integer(len),as.integer(rowlen),heights = single(len-1),hca = integer(len-1),hcb = integer(len-1), as.logical(carriageret), as.logical(verbose))
+		if(verbose){cat("\n")}
 		hcedges<-list()
 		hcedges$merge <- cbind(hcobj$hca, hcobj$hcb)
 		hcedges$height <- hcobj$heights
@@ -136,27 +143,31 @@ getLinkCommunities <- function(x, plot = TRUE, hcmethod = "average", edglim = 10
 		#return(hcedges)
 		}
 
-	hh <- unique(hcedges$height)
+	hh <- unique(round(hcedges$height, digits = 5)) # Round to 5 digits to prevent numerical instability affecting community formation.
 	countClusters <- function(x,ht){return(length(which(ht==x)))}
-	clusnums <- sapply(hh, countClusters, ht = hcedges$height) # Number of clusters at each height.
-	ldlist <- .C("getLinkDensities",as.integer(hcedges$merge[,1]), as.integer(hcedges$merge[,2]), as.integer(edges[,1]), as.integer(edges[,2]), as.integer(len), as.integer(clusnums), pdens = double(length(hh)), heights = as.double(hh), pdmax = double(1), csize = integer(1), as.logical(removetrivial), as.logical(carriageret))
+	clusnums <- sapply(hh, countClusters, ht = round(hcedges$height, digits = 5)) # Number of clusters at each height.
+	ldlist <- .C("getLinkDensities",as.integer(hcedges$merge[,1]), as.integer(hcedges$merge[,2]), as.integer(edges[,1]), as.integer(edges[,2]), as.integer(len), as.integer(clusnums), pdens = double(length(hh)), heights = as.double(hh), pdmax = double(1), csize = integer(1), as.logical(removetrivial), as.logical(carriageret), as.logical(verbose))
 	pdens <- c(0,ldlist$pdens)
 	heights <- c(0,hh)
 	pdmax <- ldlist$pdmax
 	csize <- ldlist$csize
-	
+
 	if(csize == 0){
 		stop("\nThere are no clusters appearing in this network; maybe try a larger network.\n")
 		}
 
-	cat("\n   Partition density maximum height = ",pdmax,"\n")
+	if(verbose){
+		cat("\n   Partition density maximum height = ",pdmax,"\n")
+		}
 
 	# Read in optimal clusters from a file.
 	clus <- list()
 	for(i in 1:csize){
-		mes<-paste(c("   Finishing up...1/4... ",floor((i/csize)*100),"%"),collapse="")
-		cat(mes,"\r")
-		flush.console()
+		if(verbose){
+			mes<-paste(c("   Finishing up...1/4... ",floor((i/csize)*100),"%"),collapse="")
+			cat(mes,"\r")
+			flush.console()
+			}
 		clus[[i]] <- scan(file = "linkcomm_clusters.txt", nlines = 1, skip = i-1, quiet = TRUE)
 		}
 
@@ -166,9 +177,11 @@ getLinkCommunities <- function(x, plot = TRUE, hcmethod = "average", edglim = 10
 	ecn <- data.frame()
 	ee <- data.frame()
 	for(i in 1:(length(clus))){
-		mes<-paste(c("   Finishing up...2/4... ",floor((i/length(clus))*100),"%"),collapse="")
-		cat(mes,"\r")
-		flush.console()
+		if(verbose){
+			mes<-paste(c("   Finishing up...2/4... ",floor((i/length(clus))*100),"%"),collapse="")
+			cat(mes,"\r")
+			flush.console()
+			}
 		ee <- rbind(ee,cbind(myedges[clus[[i]],],i))
 		nodes <- V(xx)$name[(unique(c(edges[clus[[i]],]))+1)]
 		both <- cbind(nodes,rep(i,length(nodes)))
@@ -180,9 +193,11 @@ getLinkCommunities <- function(x, plot = TRUE, hcmethod = "average", edglim = 10
 	ss <- NULL
 	unn <- unique(ecn[,2])
 	for(i in 1:length(unn)){
-		mes<-paste(c("   Finishing up...3/4... ",floor((i/length(unn))*100),"%"),collapse="")
-		cat(mes,"\r")
-		flush.console()
+		if(verbose){
+			mes<-paste(c("   Finishing up...3/4... ",floor((i/length(unn))*100),"%"),collapse="")
+			cat(mes,"\r")
+			flush.console()
+			}
 		ss[i] <- length(which(ecn[,2]==unn[i]))
 		}
 	names(ss) <- unn
@@ -192,12 +207,14 @@ getLinkCommunities <- function(x, plot = TRUE, hcmethod = "average", edglim = 10
 	oo<-NULL
 	unn <- unique(ecn[,1])
 	for(i in 1:length(unn)){
-		mes<-paste(c("   Finishing up...4/4... ",floor((i/length(unique(ecn[,1])))*100),"%"),collapse="")
-		cat(mes,"\r")
-		flush.console()
+		if(verbose){
+			mes<-paste(c("   Finishing up...4/4... ",floor((i/length(unique(ecn[,1])))*100),"%"),collapse="")
+			cat(mes,"\r")
+			flush.console()
+			}
 		oo[i]<-length(which(ecn[,1]==unn[i]))
 		}
-	cat("\n")
+	if(verbose){cat("\n")}
 	names(oo)<-unn
 	
 	pdplot <- cbind(heights,pdens)
@@ -221,16 +238,19 @@ getLinkCommunities <- function(x, plot = TRUE, hcmethod = "average", edglim = 10
 	all$clustsizes <- ss # Cluster sizes sorted largest to smallest (named vector where names are cluster IDs).
 	all$igraph <- xx # igraph graph.
 	all$edgelist <- el # Edge list.
+	all$directed <- directed # Logical indicating if graph is directed or not.
 
 	class(all) <- "linkcomm"
 
 	if(plot){
-		cat("   Plotting...\n")
+		if(verbose){
+			cat("   Plotting...\n")
+			}
 		if(len < 1500){ # Will be slow to plot dendrograms for large networks.
 			if(len < 500){
-				all <- plot(all, type="summary")
+				all <- plot(all, type="summary", verbose = verbose)
 			}else{
-				all <- plot(all, type="summary", right = FALSE) # Slow to reverse order of large dendrograms.
+				all <- plot(all, type="summary", right = FALSE, verbose = verbose) # Slow to reverse order of large dendrograms.
 				}
 		}else if(len <= edglim){
 			par(mfrow=c(1,2), mar=c(5.1,4.1,4.1,2.1))
